///////////////////////////////////////////////////////////////////////////////
/*! 
 *  \file GLAppGPUTest.cpp
 *  \brief FragSniffer GPU Test class.
 *  
 *  The Froggy FragSniffer.
 *	This application has been made for testing GPU fragment scheduling among 
 *  threads processors.
 *  \author C. Crassin, cyril.crassin@inrialpes.fr
 *	\date 05/03/2008
 *  \version 0.2
 */
///////////////////////////////////////////////////////////////////////////////

#include "GLAppGPUTest.h"

#include <time.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include <string>
#include <iostream>
#include <fstream>


GLAppGPUTest *GLAppGPUTest::curGPUTestApp=NULL;

bool secondQuad;

GLAppGPUTest::GLAppGPUTest(const char *title, int w, int h, int flags):
						GLApp(title, w, h, flags) {
	
}
GLAppGPUTest::GLAppGPUTest(int *argcp, char **argv, const char *title, int w, int h, int flags):
						GLApp(argcp, argv, title, w, h, flags) {
	
	currentApp=this;
	curGPUTestApp=this;
	mouseButtons[0]=mouseButtons[1]=mouseButtons[2]=0;
}

void GLAppGPUTest::keyboardSpecial(int key) {

	if(key==GLUT_KEY_END){
		moveMode=!moveMode;
	}else if(moveMode==0){ //Windows move
	switch (key) {
	case GLUT_KEY_LEFT:
		winInfo.x-=1;
		break;
	case GLUT_KEY_RIGHT:
		winInfo.x+=1;
		break;
	case GLUT_KEY_DOWN:
		winInfo.y+=1;
		break;
	case GLUT_KEY_UP:
		winInfo.y-=1;
		break;
	case GLUT_KEY_HOME:
		winInfo.z+=1;
		break;
	case GLUT_KEY_INSERT:
		winInfo.z-=1;
		break;
	case GLUT_KEY_PAGE_UP:
		winInfo.w-=1;
		break;
	case GLUT_KEY_PAGE_DOWN:
		winInfo.w+=1;
		break;
	}

	glutPositionWindow(winInfo.x, winInfo.y);
	glutReshapeWindow (winInfo.z , winInfo.w);

  }else if(moveMode==1 && curFreezePoint>=0){ //Point move
	switch (key) {
	case GLUT_KEY_LEFT:
		freezePoints[curFreezePoint].x-=1;
		break;
	case GLUT_KEY_RIGHT:
		freezePoints[curFreezePoint].x+=1;
		break;
	case GLUT_KEY_UP:
		freezePoints[curFreezePoint].y+=1;
		break;
	case GLUT_KEY_DOWN:
		freezePoints[curFreezePoint].y-=1;
		break;

	case GLUT_KEY_HOME:
		freezePoints[curFreezePoint].z+=1;
		break;
	case GLUT_KEY_INSERT:
		freezePoints[curFreezePoint].z-=1;
		break;
	case GLUT_KEY_PAGE_UP:
		freezePoints[curFreezePoint].w+=1;
		break;
	case GLUT_KEY_PAGE_DOWN:
		freezePoints[curFreezePoint].w-=1;
		break;
	}
  }

}

void GLAppGPUTest::keyboard(unsigned char key) {
if( !interfMode || !TwEventKeyboardGLUT(key,0,0) ) {

  switch (key) {
	case 'q':
		exit(0);
		break;

	case ' ':
		interfMode=!interfMode;

		if(this->interfMode){
			TwDefine(" MainOptions show ");
			this->fastMode=true;
		}else{
			this->fastMode=false;
			TwDefine(" MainOptions hide ");
		}

		break;

	case '<':
		this->controlUserPoints=!this->controlUserPoints;
		break;
	case 'u':
		loopsCount.x+=loopsCountDif.x;
		break;
	case 'j':
		loopsCount.x-=loopsCountDif.x;
		break;
	case 'i':
		loopsCount.y+=loopsCountDif.y;
		break;
	case 'k':
		loopsCount.y-=loopsCountDif.y;
		break;

	case 'U':
		loopsCountDif.x*=10;
		break;
	case 'J':
		loopsCountDif.x/=10;
		break;
	case 'I':
		loopsCountDif.y*=10;
		break;
	case 'K':
		loopsCountDif.y/=10;
		break;

	case 'y':
		miniTilesSize=miniTilesSize+Vector2i(miniTilesSizeDif);
		break;
	case 'h':
		miniTilesSize=miniTilesSize-Vector2i(miniTilesSizeDif);
		break;
	case 'Y':
		miniTilesSizeDif*=2;
		break;
	case 'H':
		miniTilesSizeDif/=2;
		break;

	case 't':
		primitivesStep+=Vector2i(1);
		break;
	case 'g':
		primitivesStep-=Vector2i(1);
		break;
	case 'T':
		primitivesStep*=Vector2i(2);
		break;
	case 'G':
		primitivesStep/=Vector2i(2);
		break;
	
	case 'p':
		primitivesSize+=Vector2i(1);
		break;
	case 'm':
		primitivesSize-=Vector2i(1);
		break;
	case 'P':
		primitivesSize*=Vector2i(2);
		break;
	case 'M':
		primitivesSize*=Vector2i(2);
		break;
		

	case 'r':
		this->gpuTestFS->reload();
		this->gpuTestVS->reload();
		break;
	case 's':
		secondQuad=!secondQuad;
		break;

	case 'x':
		fullScreen=!fullScreen;

		if(fullScreen){
			glutFullScreen();
		}else{
			glutReshapeWindow(winInfo.z, winInfo.w);
			glutPositionWindow(winInfo.x, winInfo.y);
		}

		break;
	
	case 'c':
		useBigTriangle=!useBigTriangle;
		break;

		
	case 'd':
		showInfo=!showInfo;
		break;
	case 'f':
	case '\t':
		fastMode=!fastMode;
		break;

	case 'a':
		curTest=(curTest+1)%NUM_TESTS;
		break;

	case 'v':
		samePointMode=!samePointMode;
		break;

	case 'b':
		this->blockingEnable=!this->blockingEnable;
		break;

	case 'n':
		this->lineOrientation=!this->lineOrientation;
		break;



	case '!':
		primitivesRestartMode=!primitivesRestartMode;
		break;

	case '0':
		moveMode=!moveMode;
		break;

  }

  if(moveMode==0){ //Windows move
	switch (key) {
	case '1':
		winInfo.x=0;
		winInfo.y=0;//screenSize.y-winInfo.w;
		glutPositionWindow(winInfo.x, winInfo.y);
		break;
	case '4':
		winInfo.x-=1;
		glutPositionWindow(winInfo.x, winInfo.y);
		break;
	case '6':
		winInfo.x+=1;

		glutPositionWindow(winInfo.x, winInfo.y);
		break;
	case '2':
		winInfo.y+=1;

		glutPositionWindow(winInfo.x, winInfo.y);
		break;
	case '8':
		winInfo.y-=1;

		glutPositionWindow(winInfo.x, winInfo.y);
		break;
	case '*':
		int n;
		std::cout<<"WinSize X: ";
		std::cin>>n;
		winInfo.z=n;
		std::cout<<"WinSize Y: ";
		std::cin>>n;
		winInfo.w=n;

		glutReshapeWindow(winInfo.z, winInfo.w);
		break;
	}

	

  }else if(moveMode==1 && curFreezePoint>=0){ //User move
	switch (key) {
	case '.':
		freezePoints[curFreezePoint].z=1;
		freezePoints[curFreezePoint].w=1;

		break;
	}
  }

}//ANT
}


void GLAppGPUTest::mouse(int button, int state, int x, int y){

if( !interfMode || !TwEventMouseButtonGLUT(button, state, x, y) ) {

		oldMousePos.x=x;
		oldMousePos.y=y;

		switch(button){
		case GLUT_LEFT_BUTTON:
			this->mouseButtons[0] = !state;
			break;
		case GLUT_RIGHT_BUTTON:
			this->mouseButtons[2] = !state;
			break;
		case GLUT_MIDDLE_BUTTON:
			this->mouseButtons[1] = !state;
			break;
		}


		if(this->controlUserPoints){

			//Find closest freezePoint//
			Vector2f mPos=Vector2i(x, this->windowSize.y-y);
			curUserPoint=0;

			float minDist=512.0f;
			for(int i=0; i<NUM_USER_POINTS; i++){
				if(userPoints[i].distance(mPos)<minDist ){
					minDist=userPoints[i].distance(mPos);
					curUserPoint=i;
				}
			}
		}else{
			//Find closest freezePoint//
			Vector2f mPos=Vector2i(x, this->windowSize.y-y);
			curFreezePoint=-1;

			float minDist=512.0f;
			for(int i=0; i<NUM_FREEZE_POINTS; i++){
				if(freezePoints[i].xy().distance(mPos)<minDist ){
					minDist=freezePoints[i].xy().distance(mPos);
					curFreezePoint=i;
				}
			}
		}

		

}//ANT
}


void GLAppGPUTest::mouseMove(int x, int y){
if( !interfMode || !TwEventMouseMotionGLUT(x, y) ) {

	float rx =(float)(x-oldMousePos.x);
	float ry =(float)(y-oldMousePos.y);

	Vector2i wMove=Vector2i( (x-oldMousePos.x)/1, (y-oldMousePos.y)/1);

	Vector3f trans(0.0,0.0,0.0);
	

	int px=x;
	int py=y;
	if(mouseButtons[0]){
		
		if(this->controlUserPoints){
			userPoints[curUserPoint].x=px;
			userPoints[curUserPoint].y=this->windowSize.y-py;
		}else{
			if(curFreezePoint>=0){
				Vector2i oldp2=freezePoints[curFreezePoint].xy()+freezePoints[curFreezePoint].zw();

				freezePoints[curFreezePoint].x=px;
				freezePoints[curFreezePoint].y=this->windowSize.y-py;
			}	
		}
	}else if(mouseButtons[1]){
		
	}else if(mouseButtons[2]){

		if(!this->controlUserPoints && curFreezePoint>=0){
			freezePoints[curFreezePoint].z=px-freezePoints[curFreezePoint].x;
			freezePoints[curFreezePoint].w=this->windowSize.y-py-freezePoints[curFreezePoint].y;
		}	

	}
	
	oldMousePos.x = x;
	oldMousePos.y = y;
}
}

void GLAppGPUTest::mousePassiveMove(int x, int y){

if( !interfMode || !TwEventMouseMotionGLUT(x, y) ) {
}


}


void GLAppGPUTest::reshape(int w, int h) { 

  glMatrixMode(GL_PROJECTION );
  glLoadIdentity();
  setPerspectiveProj(60.0, (float) w / (float) h, 0.01, 4.0);

  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, w, h);


   TwWindowSize(w, h); //ANT
}

void GLAppGPUTest::setPerspectiveProj( GLdouble fovY, GLdouble aspect, GLdouble zNear, GLdouble zFar ) {
	//Prespective proj
	GLdouble fW, fH;

	fH = tan( fovY / 360.0 * PI ) * zNear;//*viewSize;
	fW = fH * aspect;
	glFrustum( -fW, fW, -fH, fH, zNear, zFar );
}


void GLAppGPUTest::idle() { 

	this->redisplay();
}


void GLAppGPUTest::drawTest0(){ //Single quad

	glDrawBuffer(curClearBuffer);
	//glClearColor(0.5,0.2,0.2,0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	glDrawBuffer(curDrawBuffer);


			

#if 1
	if(useBigTriangle){
		glBegin(GL_TRIANGLES);
			glColor4f(1.0,1.0,1.0,1.0);

			glVertex4f(-10000.0 -2.0,-10000.0,-1.0,1.0);
			glVertex4f(10000.0 -2.0,-10000.0,-1.0,1.0);
			glVertex4f(10000.0 -2.0,10000.0,-1.0,1.0);
		glEnd();

	}else{
		glBegin(GL_QUADS);
		
			glColor4f(1.0,1.0,1.0,1.0);

			glVertex4f(userPoints[0].x/(this->windowSize.x*0.5f)-1.0f,userPoints[0].y/(this->windowSize.y*0.5f)-1.0f,-1.0,1.0);
			glVertex4f(userPoints[1].x/(this->windowSize.x*0.5f)-1.0f,userPoints[1].y/(this->windowSize.y*0.5f)-1.0f,-1.0,1.0);
			glVertex4f(userPoints[2].x/(this->windowSize.x*0.5f)-1.0f,userPoints[2].y/(this->windowSize.y*0.5f)-1.0f,-1.0,1.0);
			glVertex4f(userPoints[3].x/(this->windowSize.x*0.5f)-1.0f,userPoints[3].y/(this->windowSize.y*0.5f)-1.0f,-1.0,1.0);

		glEnd();
	}

#else //test
	glBegin(GL_TRIANGLES);
	
		glColor4f(1.0,1.0,1.0,1.0);

		//Same as Quad//
		/*glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);

		glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);*/


		//Test
		/*glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);*/

		Vector2f pos=Vector2f(2.0f)/(Vector2f(this->windowSize)) * (userInfo.xy())-Vector2f(1.0f);
		Vector2f pos2=Vector2f(2.0f)/(Vector2f(this->windowSize)) * (userInfo.xy()+userInfo.zw())-Vector2f(1.0f);

		glVertex4f(pos.x,pos.y,-1.0,1.0);
		glVertex4f(pos2.x,pos2.y,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);

	glEnd();
#endif
}

void GLAppGPUTest::drawTest1(){ //Single user controled triangle

	glDrawBuffer(curClearBuffer);
	//glClearColor(0.5,0.2,0.2,0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	glDrawBuffer(curDrawBuffer);


#if 1
	if(useBigTriangle){
		glBegin(GL_TRIANGLES);
			glColor4f(1.0,1.0,1.0,1.0);

			glVertex4f(-10000.0 -2.0,-10000.0,-1.0,1.0);
			glVertex4f(10000.0 -2.0,-10000.0,-1.0,1.0);
			glVertex4f(10000.0 -2.0,10000.0,-1.0,1.0);
		glEnd();

	}else{
		glBegin(GL_QUADS);
		
			glColor4f(1.0,1.0,1.0,1.0);

			glVertex4f(-1.0,-1.0,-1.0,1.0);
			glVertex4f(1.0,-1.0,-1.0,1.0);
			glVertex4f(1.0,1.0,-1.0,1.0);
			glVertex4f(-1.0,1.0,-1.0,1.0);

		glEnd();
	}

#else //test
	glBegin(GL_TRIANGLES);
	
		glColor4f(1.0,1.0,1.0,1.0);

		//Same as Quad//
		/*glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);

		glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);*/


		//Test
		/*glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);*/

		Vector2f pos=Vector2f(2.0f)/(Vector2f(this->windowSize)) * (userInfo.xy())-Vector2f(1.0f);
		Vector2f pos2=Vector2f(2.0f)/(Vector2f(this->windowSize)) * (userInfo.xy()+userInfo.zw())-Vector2f(1.0f);

		glVertex4f(pos.x,pos.y,-1.0,1.0);
		glVertex4f(pos2.x,pos2.y,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);

	glEnd();
#endif
}


void GLAppGPUTest::drawTest(){ //micro polygons 

	//Clearing//
	/*gpuTestProg->setParam("userParam", Vector4f(0.0));
	glBegin(GL_QUADS);
		glColor4f(0.0,0.0,0.0,0.0);

		glVertex4f(-1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,-1.0,-1.0,1.0);
		glVertex4f(1.0,1.0,-1.0,1.0);
		glVertex4f(-1.0,1.0,-1.0,1.0);
	glEnd();
	if(!fastMode)
		gpuTestProg->setParam("userParam", userParam);
	else
		gpuTestProg->setParam("userParam", Vector4f(0.0));*/

	glDrawBuffer(curClearBuffer);
	glClear(GL_COLOR_BUFFER_BIT);
	glDrawBuffer(curDrawBuffer);


	float qsize=float(miniTilesSize.x);
	Vector2f tileStepf=Vector2f(2.0f)/Vector2f(this->windowSize) * Vector2f(miniTilesSize); //qstep

	Vector2f primStepf=Vector2f(2.0f)/Vector2f(this->windowSize) * Vector2f(primitivesSize); //mqstep
	Vector2i primStep=this->primitivesStep;
	Vector2f primSize=Vector2f(primitivesSize)/(Vector2f(this->windowSize)*0.5f);

	if(!primitivesRestartMode){
		glBegin(primitiveType);
	}


	Vector2i curTilePos;
	Vector2f curTilePosf;

	for(int n=0; n<this->primitivesRepeat && (n==0 || primitivesRepeatMode==2); n++){

	for(curTilePosf.y=-1.0f, curTilePos.y=0; curTilePosf.y<1.0f; curTilePosf.y+=tileStepf.y, curTilePos.y+=miniTilesSize.y){
	for(curTilePosf.x=-1.0f, curTilePos.x=0; curTilePosf.x<1.0f; curTilePosf.x+=tileStepf.x, curTilePos.x+=miniTilesSize.x){
		if(!primitivesRestartMode){
			if( (curTilePos.x/miniTilesSize.x+curTilePos.y/miniTilesSize.y ) %2 )
				glColor4f(0.0,0.0,1.0,1.0);
			else
				glColor4f(0.0,1.0,0.0,1.0);
		}

		for(int n=0; n<this->primitivesRepeat && (n==0 || primitivesRepeatMode==1); n++){

		Vector2i curPrimPos;
		for(curPrimPos.y=0; curPrimPos.y<miniTilesSize.y; curPrimPos.y+=primStep.y)
		for(curPrimPos.x=0; curPrimPos.x<miniTilesSize.x; curPrimPos.x+=primStep.x){
			Vector2f curPrimPosWf=curTilePosf+Vector2f(curPrimPos)/(Vector2f(this->windowSize)*0.5f);
			
			if(samePointMode)
				curPrimPosWf=Vector2f(0.0);

			for(int n=0; n<this->primitivesRepeat && (n==0 || primitivesRepeatMode==0); n++){

				if(primitivesRestartMode){
					gpuTestProg->bind();

					glBegin(primitiveType);

					if( (curTilePos.x/miniTilesSize.x+curTilePos.y/miniTilesSize.y ) %2 )
						glColor4f(0.0,0.0,1.0,1.0);
					else
						glColor4f(0.0,1.0,0.0,1.0);
				}

			
				switch( primitiveType ){
					case GL_QUADS:
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y,				-1.0,1.0);
						glVertex4f(curPrimPosWf.x+primSize.x,	curPrimPosWf.y,				-1.0,1.0);
						glVertex4f(curPrimPosWf.x+primSize.x,	curPrimPosWf.y+primSize.y,	-1.0,1.0);
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y+primSize.y,	-1.0,1.0);
						break;
					case GL_TRIANGLES:
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y,				-1.0,1.0);
						glVertex4f(curPrimPosWf.x+primSize.x,	curPrimPosWf.y,				-1.0,1.0);
						glVertex4f(curPrimPosWf.x+primSize.x,	curPrimPosWf.y+primSize.y,	-1.0,1.0);

						glVertex4f(curPrimPosWf.x+primSize.x,	curPrimPosWf.y+primSize.y,	-1.0,1.0);
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y+primSize.y,	-1.0,1.0);
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y,				-1.0,1.0);
						break;
					case GL_POINTS:
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y,				-1.0,1.0);
						break;
					case GL_LINES:
						glVertex4f(curPrimPosWf.x,				curPrimPosWf.y,				-1.0,1.0);
						glVertex4f(curPrimPosWf.x+primSize.x,	curPrimPosWf.y+primSize.y,	-1.0,1.0);
						break;
				}
			
				if(primitivesRestartMode){
					glEnd();

					gpuTestProg->unbind();
				}

			}
		}

		}//Primitives repeat 1
	}}

	}//Primitives repeat 2

	if(!primitivesRestartMode)
		glEnd();
}

void GLAppGPUTest::display() {


	if(!fullScreen){
		winInfo.z=this->windowSize.x;
		winInfo.w=this->windowSize.y;
		winInfo.x=glutGet( GLUT_WINDOW_X );
		winInfo.y=glutGet( GLUT_WINDOW_Y );
	}

	if(interfMode){
		autoSwapBuffer=true;
		this->curDrawBuffer=GL_BACK;
		this->curClearBuffer=GL_BACK;
	}else{
		this->curDrawBuffer=GL_FRONT;
		this->curClearBuffer=GL_FRONT;

		autoSwapBuffer=false;
	}

	glDrawBuffer(this->curDrawBuffer);

	//Perspective
	//this->reshape(this->windowSize.x, this->windowSize.y);

	PerfTimer benchTimer;
	PerfTimerGL benchTimerGL;

	benchTimer.start();
	benchTimerGL.startTimer("GlobalTime");

		////Viewport clearing////
		//glViewport(0, 0, this->windowSize.x, this->windowSize.y);
		glDepthMask(GL_TRUE);
		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
		glClearColor(0.0,0.0,0.0,0.0);
		glClearDepth(1.0);
		//glClear(GL_COLOR_BUFFER_BIT);
		////////////////////////
	

		if(blockingEnable)
			this->gpuTestFS->defineMacro("ENABLE_BLOCKING", 1);
		else
			this->gpuTestFS->defineMacro("ENABLE_BLOCKING", 0);
		
		for(int i=0; i<NUM_FREEZE_POINTS; i++){
			if(freezePointsEnable[i])
				this->gpuTestFS->defineMacro( (std::string("ENABLE_FREEZE_POINT_")+i).c_str(), 1);
			else
				this->gpuTestFS->defineMacro( (std::string("ENABLE_FREEZE_POINT_")+i).c_str(), 0);
		}

		if(useBadLoop)
			this->gpuTestFS->defineMacro("USE_BAD_LOOP", 1);
		else
			this->gpuTestFS->defineMacro("USE_BAD_LOOP", 0);

		if(useBadLoop)
			this->gpuTestFS->defineMacro("NUM_PIPEVARS", numBadLoopCycles);


		gpuTestProg->bind();

		if(!fastMode)
			gpuTestProg->setParam("userParam", Vector4f(float(loopsCount.x), float(loopsCount.y), 0.0f, 0.0f) );
		else
			gpuTestProg->setParam("userParam", Vector4f(0.0));

		//gpuTestProg->setParam("userPos", userInfo);
		for(int i=0; i<NUM_FREEZE_POINTS; i++){
			gpuTestProg->setParam( (std::string("freezePoints[")+i+"]").c_str(), Vector4f(freezePoints[i]));
		}
		

		gpuTestProg->setParam("userColor", Vector4f(1.0, 1.0, 0.0, 1.0));


		switch(curTest){
			case 0:
				drawTest0();
				break;
			case 1:
				drawTest();
				break;
		}


	gpuTestProg->unbind();
	
	benchTimerGL.stopTimer();

	glFlush();
	glFinish();


	benchTimer.pushTime("Frame time", true);

	if(showInfo){
		std::cout<<"curTest: "<<curTest<<"\n";
		std::cout<<"loopsCount: "<<loopsCount<<" ("<<loopsCountDif<<")\n";
		std::cout<<"blockingEnable: "<<blockingEnable<<"\n";

		//benchTimer.displayTimes("Global Times");
		benchTimerGL.displayTimes("Global Times GL");
	}


	GLErrorCheck("Display END: ");


	///////AntTweakBar///////
	
	if(interfMode){
		glActiveTexture(GL_TEXTURE0);
		glDisable(GL_TEXTURE_RECTANGLE_ARB);
		TwDraw();
	}

	///////////////////////

	frameCounter++;

	//Buffer automatically swapped
}




void GLAppGPUTest::init(){
	this->showFPS(false);

	//
	//Glew extension manager initialisation
	glewInit();

	//Print Avalaible OpenGL extensions
	std::cout<<glGetString(GL_EXTENSIONS)<<"\n";

	int s;
	glGetIntegerv(GL_MAX_3D_TEXTURE_SIZE, &s);
	std::cout<<"MAX 3D texture size: "<<s<<"\n";

	glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS_ARB, &s);
	std::cout<<"GL_MAX_TEXTURE_IMAGE_UNITS_ARB: "<<s<<"\n";
	

	Vector3s toto;
	std::cout<<"size short vec: "<<sizeof(toto)<<"\n";

//Disable Vsync on windows plateform
#ifdef WIN32
	wglSwapIntervalEXT(1);
#endif

	PerfTimer::globalEnable();
	PerfTimerGL::globalEnable();

	/////PATHS////
	ResourcesManager::getManager()->setPath("Application:Shaders", "./Shaders/");
	/////////////


	gpuTestProg=new ShaderProgramGLSL();
	gpuTestFS=new ShaderGLSL(
			ResourcesManager::getManager()->getFullPath("Application:Shaders", "GPUTest_FS.glsl").c_str(),
			Shader::FRAGMENT_SHADER);
	gpuTestVS=new ShaderGLSL(
			ResourcesManager::getManager()->getFullPath("Application:Shaders", "GPUTest_VS.glsl").c_str(),
			Shader::VERTEX_SHADER);

	gpuTestProg->addShader(gpuTestFS);
	gpuTestProg->addShader(gpuTestVS);

	this->gpuTestFS->defineMacro("NUM_FREEZE_POINTS", NUM_FREEZE_POINTS);


	interfMode=true;

	loopsCountDif=Vector2i(1000000, 100000);
	loopsCount=loopsCountDif*2;


	//userInfo=Vector4f(0.0,0.0, 0.0, 1.0);

	for(int i=0; i<NUM_FREEZE_POINTS; i++){
		freezePoints[i]=Vector4i(0, 0, 1, 1);

		freezePointsEnable[i]=false;
	}
	freezePointsEnable[0]=true;
	curFreezePoint=0;

	userPoints[0]=Vector2i(0,0);
	userPoints[1]=Vector2i(this->windowSize.x,0);
	userPoints[2]=Vector2i(this->windowSize.x,this->windowSize.y);
	userPoints[3]=Vector2i(0,this->windowSize.y);
	curUserPoint=0;
	controlUserPoints=false;


	useBadLoop=false;
	numBadLoopCycles=20;

	screenSize=Vector2i(1680,1050);

	winInfo=Vector4i(512,512, 512, 512);
	glutPositionWindow(winInfo.x, winInfo.y);
	glutReshapeWindow(winInfo.z, winInfo.w);

	secondQuad=true;

	
	curTest=0;
	miniTilesSize=32;
	miniTilesSizeDif=1;

	frameCounter=0;

	primitiveType=GL_QUADS;
	primitivesSize=Vector2i(8);
	primitivesStep=primitivesSize;
	primitivesRepeat=1;
	primitivesRepeatMode=0;


	samePointMode=0;

	lineOrientation=0;

	fastMode=true;
	fullScreen=true;
	moveMode=1;
	useBigTriangle=false;
	blockingEnable=true;

	primitivesRestartMode=false;


	//States setting
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_STENCIL_TEST);
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_BLEND);


	this->curDrawBuffer=GL_FRONT;
	this->curClearBuffer=GL_BACK;
	autoSwapBuffer=false;

	//////AntTweakBar//////
	TwInit(TW_OPENGL, NULL);
	TwWindowSize(this->windowSize.x, this->windowSize.y);
	TwGLUTModifiersFunc(glutGetModifiers);

	TwDefine(" GLOBAL help='This Froggy FragSniffer has been made to allow deep looking into GPUs rasterisation and fragments scheduling among computing ressources.\
		\n\n Use the [space bar] to switch between test mode and configuration mode.\n In test mode, you can enable slow down mode by pressing [TAB].\
		\n Total rendering time of a frame is displayed in the console window as the loops control configuration.\
		\n Activated freezing pixels can be controled by mouse. Dragging red pixels with left mouse button will set the position, the right button will set its size. Use [b] or the interface button to disable freezing.\
		\n You can switch between a Pixel keyboard control mode (controling active freezing pixel) and a Window keyboard control mode (controling window position) pressing [END].\
		\n In each mode, position and size can be controled precisely by keyboard through the [LEFT], [RIGHT], [TOP], [BOTTOM] directions keys for position and [INSERT], [HOME], [PAGE UP], [PAGE DOWN] for size.\
		\n You can switch between a large space filling primitive and a multiple primitives modes pressing [a].\
		\n Drawn primitives can be chosen fron the interface in config mode.\
		\n Normal fragments and freezed fragments speed can be changed within each configuration bar through the number of loop cycles entry.' ");

	//TwDefine(" GLOBAL show ");
	//TwDefine(" GLOBAL color='128 128 0' ");

	optBarMain = TwNewBar("MainOptions");
	TwDefine(" MainOptions position='350 300' size='250 260' fontSize='1'"); 
	TwDefine(" MainOptions help='Get help clicking on the bottom left help icon.' ");

	TwAddVarRW(optBarMain, "Fast Mode [TAB]", TW_TYPE_BOOLCPP, &fastMode, "");
	
	// Add the enum variable 'curTest' to 'bar'
	// (before adding an enum variable, its enum type must be declared to AntTweakBar as follow)
	{
		// ShapeEV associates Shape enum values with labels that will be displayed instead of enum values
		//TwEnumVal curTestEV[NUM_TESTS] = { {0, "Big prim."}, {1, "Quads"}, {2, "Points"}, {3, "Micro polys"}, {4, "Lines"}, {5, "Points"} };
		TwEnumVal curTestEV[NUM_TESTS] = { {0, "Big prim."}, {1, "Small primitives"} };
		// Create a type for the enum shapeEV
		TwType curTestType = TwDefineEnum("curTest", curTestEV, NUM_TESTS);
		// add 'g_CurrentShape' to 'bar': it is a variable of type ShapeType. Its key shortcuts are [<] and [>].
		TwAddVarRW(optBarMain, "Active Test [a]", curTestType, &curTest, " keyIncr='a' keyDecr='A' help='Change object shape.' ");
	}
	//TwAddVarRW(optBarMain, "Active Test [b]", TW_TYPE_INT32, &curTest, "");

	{
		TwEnumVal curPrimEV[4] = { {GL_QUADS, "Quads"}, {GL_TRIANGLES, "Triangles"}, {GL_LINES, "Lines"}, {GL_POINTS, "Points"} };
		TwType curPrimType = TwDefineEnum("curPrim", curPrimEV, 4);
		TwAddVarRW(optBarMain, "Primitive type", curPrimType, &primitiveType, " ");
	}

	
	//Types//
	TwStructMember vec2fMembers[] = { 
		{ "x", TW_TYPE_FLOAT, offsetof(Vector2f, x), " Min=-1 Max=1 Step=0.01 " },
		{ "y", TW_TYPE_FLOAT, offsetof(Vector2f, y), " Min=-1 Max=1 Step=0.01 " } };
	TwType vec2fType = TwDefineStruct("Vec2f", vec2fMembers, 2, sizeof(Vector2f), NULL, NULL);
	TwStructMember vec2iMembers[] = { 
		{ "x", TW_TYPE_INT32, offsetof(Vector2i, x), " Min=0 Max=2048 Step=1 " },
		{ "y", TW_TYPE_INT32, offsetof(Vector2i, y), " Min=0 Max=2048 Step=1 " } };
	TwType vec2iType = TwDefineStruct("Vec2i", vec2iMembers, 2, sizeof(Vector2i), NULL, NULL);
	TwStructMember vec4iMembers[] = { 
		{ "x", TW_TYPE_INT32, offsetof(Vector4i, x), " Min=0 Max=2048 Step=1 " },
		{ "y", TW_TYPE_INT32, offsetof(Vector4i, y), " Min=0 Max=2048 Step=1 " }, 
		{ "z", TW_TYPE_INT32, offsetof(Vector4i, z), " Min=0 Max=2048 Step=1 " }, 
		{ "w", TW_TYPE_INT32, offsetof(Vector4i, w), " Min=0 Max=2048 Step=1 " } };
	TwType vec4iType = TwDefineStruct("Vec4i", vec4iMembers, 2, sizeof(Vector4i), NULL, NULL);


	TwAddVarRW(optBarMain, "Slowdown loop cycles std [i/k] (Increment [I/K])", TW_TYPE_INT32, &loopsCount.y, "help='Number of cycles of the slowdown loop for standard pixels'");

	TwAddVarRW(optBarMain, "Primitives tilling size [y/h]", vec2iType, &miniTilesSize, "help='Size of primitives tilling'");

	TwAddVarRW(optBarMain, "Primitives size", vec2iType, &primitivesSize, "help='Size in pixels of each primitives'");
	TwAddVarRW(optBarMain, "Primitives step [t/g]", vec2iType, &primitivesStep, "help='Space in pixels between two primitives positions'");
	
	TwAddVarRW(optBarMain, "Primitives redraw", TW_TYPE_INT32, &primitivesRepeat, "");
	{
		TwEnumVal repPrimEV[3] = { {0, "Primitive"}, {1, "Tiles"}, {2, "Frame"} };
		TwType repPrimType = TwDefineEnum("repPrim", repPrimEV, 3);
		TwAddVarRW(optBarMain, "Primitive redraw mode", repPrimType, &primitivesRepeatMode, "help='Chose the type of thing to redraw'");
	}

	//TwAddVarRW(optBarMain, "Primitives redraw mode", TW_TYPE_BOOLCPP, &primitivesRepeatMode, "");
	

	TwAddVarRW(optBarMain, "All on one pixel", TW_TYPE_BOOLCPP, &samePointMode, "help='All primitives of a configuration are drawn on the same point'");
	
	TwAddVarRW(optBarMain, "Draw call per primitive", TW_TYPE_BOOLCPP, &primitivesRestartMode, "help='Will use a different draw call for each primitive'");

	TwAddVarRW(optBarMain, "Big triangle [c]", TW_TYPE_BOOLCPP, &useBigTriangle, "help='Draw a a fully clipped triangle in big primitive mode'");
	TwAddVarRW(optBarMain, "Control big primitive [<]", TW_TYPE_BOOLCPP, &controlUserPoints, "help='Control vertices of the primitive in big primitive mode'");
	
	
	/////////Points
	

	optBarPoints = TwNewBar("FreezyPoints");
	TwDefine(" FreezyPoints position='350 0' size='200 300' fontSize='1'  valuesWidth='80' "); 

	TwAddVarRW(optBarPoints, "Pixels Blocking [b]", TW_TYPE_BOOLCPP, &blockingEnable, "");

	TwAddVarRW(optBarPoints, "Use reg. saturation", TW_TYPE_BOOLCPP, &useBadLoop, "");
	TwAddVarRW(optBarPoints, "Num shader regs.", TW_TYPE_INT32, &numBadLoopCycles, "");

	TwAddVarRO(optBarPoints, "Current Point", TW_TYPE_INT32, &curFreezePoint, "");

	TwAddVarRW(optBarPoints, "Slowdown loop cycles [u/j] (Increment [U/J])", TW_TYPE_INT32, &loopsCount.x, "help='Number of cycles of the slowdown loop for freezing pixels'");
	
	{
		TwEnumVal moveModeEV[2] = { {0, "Window"}, {1, "Freeze points"} };
		TwType moveModeType = TwDefineEnum("moveMode", moveModeEV, 2);
		TwAddVarRW(optBarPoints, "Keyboard move ctrl.", moveModeType, &moveMode, " ");
	}

	

		
	TwStructMember pointMembers[] = { 
		{ "PosX", TW_TYPE_INT32, offsetof(Vector4i, x), " Min=0 Max=2048 Step=1 " },
		{ "PosY", TW_TYPE_INT32, offsetof(Vector4i, y), " Min=0 Max=2048 Step=1 " },
		{ "SizeX", TW_TYPE_INT32, offsetof(Vector4i, z), " Min=0 Max=2048 Step=1 " },
		{ "SizeY", TW_TYPE_INT32, offsetof(Vector4i, w), " Min=0 Max=2048 Step=1 " } };
	TwType pointType = TwDefineStruct("Point", pointMembers, 4, sizeof(Vector4i), NULL, NULL);


	for(int i=0; i<NUM_FREEZE_POINTS; i++){
		freezePointsNum[i]=i;
		TwAddSeparator(optBarPoints, (std::string("Point ")+i).c_str(), "");
		TwAddVarRW(optBarPoints, (std::string("Enable P")+i).c_str(), TW_TYPE_BOOLCPP, &freezePointsEnable[i], "");
		TwAddVarRW(optBarPoints, (std::string("Parameters P")+i).c_str() , pointType, &freezePoints[i], "");
		TwAddButton(optBarPoints, (std::string("Unit Size P")+i).c_str(), callbackPUS, &freezePointsNum[i], "");
	}

	GLErrorCheck("Init END: ");
}