///////////////////////////////////////////////////////////////////////////////
/*! \file Lighting.h
 *  \brief Framework Nemo
 *  \author C. Crassin
 *
 * $Id: ShaderProgram.h,v 1.2 2007/09/14 11:54:56 gascuel Exp $
 */
///////////////////////////////////////////////////////////////////////////////
// History:
//
// 2005-01-01 : Creation.

#ifndef ShaderProgram_h
#define ShaderProgram_h

#include "UtilsGL.h"
#include "Vector.h"
#include "Matrix.h"

#include <vector>
#include <list>

namespace NemoGraphics {

class Texture;

class Shader {	
public:
	enum ShaderType{VERTEX_SHADER=0, FRAGMENT_SHADER=1, GEOMETRY_SHADER=2};

protected:
	/** Shader loading from file.*/
	static std::string loadSource(const char *name);

	std::string sourceFileName;

	std::string shaderSource;

	std::list<Shader *> subShaders;

	bool sourceChanged;

	ShaderType shaderType;
public:

	virtual ~Shader(){}

	virtual void defineMacro(const char *macro, const char *value)=0;
	virtual void defineMacro(const char *macro, int val)=0;
	virtual void defineMacro(const char *macro, float val)=0;

	virtual bool compile()=0;

	std::string *getSource(){
		return &shaderSource;
	}
	std::string &getSourceFileName(){
		return sourceFileName;
	}

	ShaderType getType(){
		return shaderType;
	}

	virtual void attachModule(Shader *shad){
		subShaders.push_back(shad);
		sourceChanged=true;
	}
	virtual void detachModule(Shader *shad){
		subShaders.remove(shad);
		sourceChanged=true;
	}

	virtual void reload(){}
};

class ShaderProgram {

public:

	virtual ~ShaderProgram(){
	}

	virtual void bind(bool render=true)=0;
	virtual void unbind(bool render=true)=0;

	void addShader(Shader &shader){
		addShader(&shader);
	}
	virtual void addShader(Shader *shader)=0;


	virtual int getProgramID()=0;
	//Parameters setting//
	virtual void setParam(const char *pname, float val)=0;

	virtual void setParam(const char *pname, const Vector2f &val)=0;
	virtual void setParam(const char *pname, const Vector3f &val)=0;
	virtual void setParam(const char *pname, const Vector4f &val)=0;

	virtual void setParam(const char *pname, int v)=0;
	virtual void setParam(const char *pname, const Vector3i &val)=0;

	virtual void setParam(const char *pname, const Mat4f &val)=0;

	///virtual void setSamplerParam(const char *pname, GLuint texid, GLuint stype=0)=0;
	virtual void setSamplerParam(const char *pname, Texture *tex)=0;

	virtual void setFragDataLocation(const char *pname, GLuint colnum)=0;


	////////////////
	virtual void setAttrib(const char *aname, Vector3f *data){}
	virtual void setAttrib(const char *aname, Vector4f *data){}
};

};
#endif
